# Generated by Django 2.2.8 on 2020-02-19 14:05

from django.db import migrations
import scoped_rbac.fields


class Migration(migrations.Migration):

    dependencies = [("scoped_rbac", "0001_initial")]

    operations = [
        migrations.RenameField(
            model_name="role", old_name="rbac_context", new_name="context"
        ),
        migrations.RenameField(
            model_name="roleassignment", old_name="rbac_context", new_name="context"
        ),
        migrations.RemoveField(model_name="role", name="definition_json"),
        migrations.AddField(
            model_name="role",
            name="definition",
            field=scoped_rbac.fields.JSONField(default={}),
            preserve_default=False,
        ),
    ]
