# Generated by Django 2.2.8 on 2020-02-23 18:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("scoped_rbac", "0002_auto_20200219_1405")]

    operations = [
        migrations.RemoveField(model_name="role", name="context"),
        migrations.RemoveField(model_name="roleassignment", name="context"),
        migrations.DeleteModel(name="Context"),
    ]
